from .payments import PaypalPDT, PaypalIPN


def pdt_confirm(transaction_id, token, sandbox=False, endpoint=''):
    """
    API - confirm a transaction via Paypal PDT and return
    a PaypalResponse instance with the processed response data
    """
    pdt = PaypalPDT(transaction_id, token, sandbox=sandbox, endpoint=endpoint)
    pdt.send_confirmation()
    pdt.process_response()
    return pdt.to_paypal_response()


def ipn_confirm(query_string, sandbox=False, endpoint=''):
    """
    API - confirm a transaction via Paypal IPN and return
    a PaypalResponse instance with the processed response data
    """
    ipn = PaypalIPN(query_string, sandbox=sandbox, endpoint=endpoint)
    ipn.send_confirmation()
    ipn.process_response()
    return ipn.to_paypal_response()



__title__ = 'pypay'
__version__ = '0.2.1'
__version_info__ = tuple(__version__.split('.'))
__author__ = 'Judd Garratt'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2017 Judd Garratt'

from .api import pdt_confirm, ipn_confirm



from .exceptions import InvalidPaypalData, RequestError
from six.moves import urllib
import requests
import six


class PaypalPayments(object):
    """
    Base Paypal payments class. Defines endpoints and allows a processed
    Paypal response to be conferted into a PaypalResponse object
    """

    live_endpoint = 'https://www.paypal.com/cgi-bin/webscr'
    sandbox_endpoint = 'https://www.sandbox.paypal.com/cgi-bin/webscr'

    sandbox = False
    endpoint = ''
    response = None
    confirmed = False
    details = None

    def send_confirmation(self):
        raise NotImplementedError()

    def process_response(self):
        raise NotImplementedError()

    def to_paypal_response(self):
        return PaypalResponse(self.confirmed, self.details)


class PaypalResponse(object):
    """
    Wrapper around Paypal response data for more consistent api
    """

    def __init__(self, confirmed, details):
        self.confirmed = confirmed
        self.details = details


class PaypalPDT(PaypalPayments):
    """
    For a given TransactionID confirm a payment via Paypal PDT and
    process the response
    """

    def __init__(self, transaction_id, identity_token, sandbox=False, endpoint=''):
        if not transaction_id or not isinstance(transaction_id, six.string_types):
            raise InvalidPaypalData('{0} is not a valid Paypal TransactionID'.format(transaction_id))
        if not identity_token or not isinstance(identity_token, six.string_types):
            raise InvalidPaypalData('{0} is not a valid Paypal identity token'.format(identity_token))
        self.transaction_id = transaction_id
        self.identity_token = identity_token
        self.sandbox = sandbox
        if endpoint:
            self.endpoint = endpoint
        else:
            self.endpoint = self.live_endpoint if not self.sandbox else self.sandbox_endpoint

    def send_confirmation(self):
        params = {'cmd': '_notify-synch', 'tx': self.transaction_id, 'at': self.identity_token}
        try:
            self.response = requests.post(self.endpoint, params=params)
        except requests.exceptions.RequestException as e:
            raise RequestError(e.args[0])
        if self.response.status_code != 200:
            raise RequestError('Paypal returned a {0} status'.format(self.response.status_code))

    def process_response(self):
        if self.response:
            data = self.response.text.strip().split('\n')
            self.confirmed = True if data[0].strip() == 'SUCCESS' else False
            self.details = {}
            for field in data[1:]:
                field = field.split('=', 1)
                if len(field) != 2:
                    continue
                name = field[0]
                value = field[1]
                self.details[name] = urllib.parse.unquote_plus(value.strip())


class PaypalIPN(PaypalPayments):
    """
    For a given set of query params confirm a payment via Paypal IPN and
    process the response
    """

    def __init__(self, query_params, sandbox=False, endpoint=''):
        if isinstance(query_params, six.string_types):
            self.query_params = query_params
        elif isinstance(query_params, dict):
            self.query_params = urllib.parse.urlencode(query_params)
        else:
            raise InvalidPaypalData('{0} is not a valid IPN query string'.format(query_params))
        self.sandbox = sandbox
        if endpoint:
            self.endpoint = endpoint
        else:
            self.endpoint = self.live_endpoint if not self.sandbox else self.sandbox_endpoint

    def send_confirmation(self):
        headers = {'content_type': 'x-www-form-urlencoded'}
        post_url = '{0}?{1}&{2}'.format(self.endpoint, 'cmd=_notify-validate', self.query_params)
        try:
            self.response = requests.post(post_url, headers=headers)
        except requests.exceptions.RequestException as e:
            raise RequestError(e.args[0])
        if self.response.status_code != 200:
            raise RequestError('Paypal returned a {0} status'.format(self.response.status_code))

    def process_response(self):
        if self.response:
            if self.response.text.strip() == 'VERIFIED':
                self.confirmed = True
            else:
                self.confirmed = False
        self.details = dict((k, v[0].strip()) for (k, v) in urllib.parse.parse_qs(self.query_params, keep_blank_values=True).items()) # no dict comprehensions in Python 2.6 :(



class InvalidPaypalData(Exception):
    """
    Raised when data to be sent to Paypal fails validation
    """

class RequestError(Exception):
    """
    Raised when something goes wrong connecting to Paypal
    """
    pass



#!/usr/bin/env python

import re
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('pypay/__init__.py', 'rU') as pkg_file:
    version = re.search(r'__version__ = \'([0-9\.]+)\'\n', pkg_file.read()).group(1)

with open('README.rst', 'rU') as readme_file:
    readme = readme_file.read()

setup(
    name='pypay',
    version=version,
    description='Confirm Paypal payments via PDT and IPN',
    long_description=readme,
    author='Judd Garratt',
    author_email='judd.garratt@gmail.com',
    url='http://bitbucket.org/juddgarratt/pypay',
    packages=['pypay'],
    package_data={'': ['LICENSE']},
    package_dir={'pypay': 'pypay'},
    include_package_data=True,
    install_requires=['requests', 'six'],
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
